package net.knarcraft.blockhunt.arena;

import me.libraryaddict.disguise.DisguiseAPI;
import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.PlayerArenaData;
import net.knarcraft.blockhunt.PlayerHandler;
import net.knarcraft.blockhunt.ScoreboardHandler;
import net.knarcraft.blockhunt.SignsHandler;
import net.knarcraft.blockhunt.config.ConfigKey;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.config.Permission;
import net.knarcraft.blockhunt.event.EndArenaEvent;
import net.knarcraft.blockhunt.event.JoinArenaEvent;
import net.knarcraft.blockhunt.event.LeaveArenaEvent;
import net.knarcraft.blockhunt.manager.PermissionsManager;
import net.knarcraft.blockhunt.util.MessageHelper;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.attribute.Attribute;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.potion.PotionEffect;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

public class ArenaHandler {

    public static void loadArenas() {
        MemoryStorage.arenaMap.clear();
        for (String arenaName : MemoryStorage.arenas.getFileConfiguration().getKeys(false)) {
            MemoryStorage.arenaMap.put(arenaName.toLowerCase(), (Arena) MemoryStorage.arenas.getFileConfiguration().get(arenaName));
        }

        for (Arena arena : MemoryStorage.arenaMap.values()) {
            ScoreboardHandler.createScoreboard(arena);
        }
    }

    /**
     * Sends a message to all players in an arena
     *
     * @param arena     <p>The arena to announce to</p>
     * @param message   <p>The message to display</p>
     * @param variables <p>The variables to replace in the message</p>
     */
    public static void sendMessage(Arena arena, String message, String... variables) {
        MessageHelper.broadcastMessage(arena.playersInArena, message, variables);
    }

    /**
     * Sends a message to all players in an arena
     *
     * @param arena      <p>The arena to announce to</p>
     * @param messageKey <p>The message key defining the string to display</p>
     * @param variables  <p>The variables to replace in the message</p>
     */
    public static void sendMessage(Arena arena, MessageKey messageKey, String... variables) {
        MessageHelper.broadcastMessage(arena.playersInArena, messageKey, variables);
    }

    public static void playerJoinArena(Player player, String arenaName) {
        boolean alreadyJoined = false;
        for (Arena arena : MemoryStorage.arenaMap.values()) {
            if (arena.playersInArena != null && arena.playersInArena.contains(player)) {
                alreadyJoined = true;
                break;
            }
        }

        if (alreadyJoined) {
            MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_ALREADY_JOINED);
            return;
        }

        Arena arena = MemoryStorage.arenaMap.get(arenaName.toLowerCase());
        if (arena != null) {
            if (arena.disguiseBlocks.isEmpty()) {
                MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_NO_BLOCKS_SET);
            } else {
                boolean inventoryEmpty = true;
                for (ItemStack inventoryItem : player.getInventory()) {
                    if (inventoryItem != null && inventoryItem.getType() != Material.AIR) {
                        inventoryEmpty = false;
                    }
                }

                for (ItemStack inventoryItem : player.getInventory().getArmorContents()) {
                    if (inventoryItem != null && inventoryItem.getType() != Material.AIR) {
                        inventoryEmpty = false;
                    }
                }

                if ((Boolean) MemoryStorage.config.get(ConfigKey.REQUIRE_INVENTORY_CLEAR_ON_JOIN) && !inventoryEmpty) {
                    MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_INVENTORY_NOT_EMPTY);
                    return;
                }

                Location zero = new Location(Bukkit.getWorld(player.getWorld().getName()), 0, 0, 0, 0, 0);
                ArenaLocations locations = arena.arenaLocations;
                if (locations.getLobbyWarp() != null && locations.getHidersWarp() != null &&
                        locations.getSeekersWarp() != null && locations.getExitWarp() != null) {
                    if (!locations.getLobbyWarp().equals(zero) && !locations.getHidersWarp().equals(zero) &&
                            !locations.getSeekersWarp().equals(zero) && !locations.getExitWarp().equals(zero)) {
                        if (arena.gameState == ArenaState.WAITING || arena.gameState == ArenaState.STARTING) {
                            if (arena.playersInArena.size() >= arena.maxPlayers &&
                                    !PermissionsManager.hasPermission(player, Permission.JOIN_FULL, false)) {
                                MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_FULL);
                                return;
                            }

                            boolean canWarp = PlayerHandler.teleport(player, locations.getLobbyWarp());
                            if (!canWarp) {
                                MessageHelper.sendMessage(player, MessageKey.ERROR_TELEPORT_FAILED);
                                return;
                            }

                            BlockHunt.plugin.getLogger().log(Level.INFO, player.getName() + " has joined " + arenaName);
                            arena.playersInArena.add(player);
                            JoinArenaEvent event = new JoinArenaEvent(player, arena);
                            Bukkit.getPluginManager().callEvent(event);

                            PlayerArenaData pad = new PlayerArenaData(player.getLocation(), player.getGameMode(), player.getInventory().getContents(), player
                                    .getInventory().getArmorContents(), player.getExp(), player.getLevel(), player.getAttribute(Attribute.GENERIC_MAX_HEALTH).getValue(), player.getHealth(), player.getFoodLevel(),
                                    player.getActivePotionEffects(), player.getAllowFlight());

                            MemoryStorage.playerData.put(player, pad);

                            player.setGameMode(GameMode.SURVIVAL);
                            for (PotionEffect pe : player.getActivePotionEffects()) {
                                player.removePotionEffect(pe.getType());
                            }
                            player.setFoodLevel(20);
                            player.setHealth(20);
                            player.getAttribute(Attribute.GENERIC_MAX_HEALTH).setBaseValue(20);
                            player.setLevel(arena.timer);
                            player.setExp(0);
                            player.getInventory().clear();
                            player.getInventory().setHelmet(new ItemStack(Material.AIR));
                            player.getInventory().setChestplate(new ItemStack(Material.AIR));
                            player.getInventory().setLeggings(new ItemStack(Material.AIR));
                            player.getInventory().setBoots(new ItemStack(Material.AIR));
                            player.setFlying(false);
                            player.setAllowFlight(false);
                            player.setWalkSpeed(0.2F);

                            // Fix for client not showing players after
                            // they join
                            for (Player otherPlayer : arena.playersInArena) {
                                if (otherPlayer.canSee(player)) {
                                    otherPlayer.showPlayer(BlockHunt.plugin, player); // Make
                                }
                                // new
                                // player
                                // visible
                                // to
                                // others
                                if (player.canSee(otherPlayer)) {
                                    player.showPlayer(BlockHunt.plugin, otherPlayer); // Make
                                }
                                // other
                                // players
                                // visible
                                // to
                                // new
                                // player
                            }

                            if ((Boolean) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_ENABLED)) {
                                if (MemoryStorage.shop.getFileConfiguration().get(player.getName() + ".blockchooser") != null
                                        || PermissionsManager.hasPermission(player, Permission.SHOP_BLOCK_CHOOSER, false)) {
                                    ItemStack shopBlockChooser = new ItemStack(Material.getMaterial((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_ID_NAME)), 1);
                                    ItemMeta shopBlockChooser_IM = shopBlockChooser.getItemMeta();
                                    shopBlockChooser_IM.setDisplayName(MessageHelper.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME)));
                                    List<String> loreStrings = MemoryStorage.config.getFileConfiguration().getStringList(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_DESCRIPTION.getPath());
                                    List<String> loreStrings2 = new ArrayList<>();
                                    for (String lore : loreStrings) {
                                        loreStrings2.add(MessageHelper.replaceAll(lore));
                                    }
                                    shopBlockChooser_IM.setLore(loreStrings2);
                                    shopBlockChooser.setItemMeta(shopBlockChooser_IM);

                                    player.getInventory().addItem(shopBlockChooser);
                                }
                            }

                            if ((Boolean) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_ENABLED)) {
                                if (MemoryStorage.shop.getFileConfiguration().getInt(player.getName() + ".blockhuntpass") != 0) {
                                    ItemStack shopBlockHuntPass = new ItemStack(Material.getMaterial((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_ID_NAME)),
                                            1);
                                    ItemMeta shopBlockHuntPass_IM = shopBlockHuntPass.getItemMeta();
                                    shopBlockHuntPass_IM.setDisplayName(MessageHelper.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME)));
                                    List<String> loreStrings = MemoryStorage.config.getFileConfiguration().getStringList(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_DESCRIPTION.getPath());
                                    List<String> loreStrings2 = new ArrayList<>();
                                    for (String lore : loreStrings) {
                                        loreStrings2.add(MessageHelper.replaceAll(lore));
                                    }

                                    shopBlockHuntPass_IM.setLore(loreStrings2);
                                    shopBlockHuntPass.setItemMeta(shopBlockHuntPass_IM);
                                    shopBlockHuntPass.setAmount(MemoryStorage.shop.getFileConfiguration().getInt(player.getName() + ".blockhuntpass"));

                                    player.getInventory().addItem(shopBlockHuntPass);
                                }
                            }

                            DisguiseAPI.undisguiseToAll(player);

                            ArenaHandler.sendMessage(arena, MessageKey.NORMAL_JOIN_JOINED_ARENA, "%playerName%-" + player.getName(),
                                    "1-" + arena.playersInArena.size(), "2-" + arena.maxPlayers);
                            if (arena.playersInArena.size() < arena.minPlayers) {
                                sendMessage(arena, MessageKey.WARNING_LOBBY_NEED_AT_LEAST, "1-" + arena.minPlayers);
                            }
                        } else {
                            MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_ARENA_IN_GAME);
                        }
                    } else {
                        MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_WARPS_NOT_SET);
                    }
                } else {
                    MessageHelper.sendMessage(player, MessageKey.ERROR_JOIN_WARPS_NOT_SET);
                }
            }
        } else {
            MessageHelper.sendMessage(player, MessageKey.ERROR_NO_ARENA, "name-" + arenaName);
        }

        SignsHandler.updateSigns();
    }

    public static void playerLeaveArena(Player player, boolean message, boolean cleanup) {

        Arena arena = null;
        for (Arena arena2 : MemoryStorage.arenaMap.values()) {
            if (arena2.playersInArena != null) {
                if (arena2.playersInArena.contains(player)) {
                    arena = arena2;
                }
            }
        }

        if (arena != null) {
            BlockHunt.plugin.getLogger().log(Level.INFO, player.getName() + " has left " + arena.arenaName);
            LeaveArenaEvent event = new LeaveArenaEvent(player, arena);
            Bukkit.getPluginManager().callEvent(event);

            if (cleanup) {
                arena.playersInArena.remove(player);
                arena.seekers.remove(player);
                arena.hiders.remove(player);

                if (arena.playersInArena.size() < arena.minPlayers && arena.gameState.equals(ArenaState.STARTING)) {
                    arena.gameState = ArenaState.WAITING;
                    arena.timer = 0;

                    sendMessage(arena, MessageKey.WARNING_LOBBY_NEED_AT_LEAST, "1-" + arena.minPlayers);
                }

                if (arena.playersInArena.size() <= 1 && arena.gameState == ArenaState.IN_GAME) {
                    if (arena.seekers.size() >= arena.playersInArena.size()) {
                        ArenaHandler.seekersWin(arena);
                    } else {
                        ArenaHandler.hidersWin(arena);
                    }
                }

                if (arena.seekers.size() >= arena.playersInArena.size()) {
                    ArenaHandler.seekersWin(arena);
                }

                if (arena.seekers.size() == 0 && arena.gameState == ArenaState.IN_GAME) {
                    Player seeker = arena.playersInArena.get(MemoryStorage.random.nextInt(arena.playersInArena.size()));
                    ArenaHandler.sendMessage(arena, MessageKey.WARNING_IN_GAME_NEW_SEEKER_CHOSEN, "seeker-" + seeker.getName());
                    ArenaHandler.sendMessage(arena, MessageKey.NORMAL_IN_GAME_SEEKER_CHOSEN, "seeker-" + seeker.getName());
                    DisguiseAPI.undisguiseToAll(seeker);
                    for (Player pl : Bukkit.getOnlinePlayers()) {
                        pl.showPlayer(BlockHunt.plugin, seeker);
                    }
                    seeker.getInventory().clear();
                    arena.seekers.add(seeker);
                    PlayerHandler.teleport(seeker, arena.arenaLocations.getSeekersWarp());
                    MemoryStorage.seekerTime.put(seeker, arena.waitingTimeSeeker);
                    seeker.setWalkSpeed(0.3F);

                    // Fix for client not showing players after they join
                    for (Player otherPlayer : arena.playersInArena) {
                        if (otherPlayer.canSee(player)) {
                            otherPlayer.showPlayer(BlockHunt.plugin, player); // Make new player
                        }
                        // visible to others
                        if (player.canSee(otherPlayer)) {
                            player.showPlayer(BlockHunt.plugin, otherPlayer); // Make other
                        }
                        // players visible
                        // to new player
                    }
                }
            }

            PlayerArenaData pad = new PlayerArenaData(null, null, null, null, null, null, null, null, null, null, false);

            if (MemoryStorage.playerData.get(player) != null) {
                pad = MemoryStorage.playerData.get(player);
            }

            player.getInventory().clear();
            player.getInventory().setContents(pad.playerInventory());
            player.getInventory().setArmorContents(pad.playerArmor());
            player.setExp(pad.playerEXP());
            player.setLevel(pad.playerEXPLevels());
            player.getAttribute(Attribute.GENERIC_MAX_HEALTH).setBaseValue(pad.playerMaxHealth());
            player.setHealth(pad.playerHealth());
            player.setFoodLevel(pad.playerFood());
            player.addPotionEffects(pad.playerPotionEffects());
            PlayerHandler.teleport(player, arena.arenaLocations.getExitWarp());
            player.setGameMode(pad.playerGameMode());
            player.setAllowFlight(pad.playerFlying());
            if (player.getAllowFlight()) {
                player.setFlying(true);
            }
            player.setWalkSpeed(0.2F);

            MemoryStorage.playerData.remove(player);
            MemoryStorage.chosenBlock.remove(player);

            for (Player pl : Bukkit.getOnlinePlayers()) {
                pl.showPlayer(BlockHunt.plugin, player);
                if (MemoryStorage.hiddenLocation.get(player) != null) {
                    if (MemoryStorage.hiddenLocationWater.get(player) != null) {
                        Block pBlock = MemoryStorage.hiddenLocation.get(player).getBlock();
                        if (MemoryStorage.hiddenLocationWater.get(player)) {
                            pl.sendBlockChange(pBlock.getLocation(), Bukkit.createBlockData(Material.WATER));
                        } else {
                            pl.sendBlockChange(pBlock.getLocation(), Bukkit.createBlockData(Material.AIR));
                        }
                    }
                }

                DisguiseAPI.undisguiseToAll(player);
            }

            ScoreboardHandler.removeScoreboard(player);

            MessageHelper.sendMessage(player, MessageKey.NORMAL_LEAVE_YOU_LEFT);
            if (message) {
                ArenaHandler.sendMessage(arena, MessageKey.NORMAL_LEAVE_LEFT_ARENA, "%playerName%-" + player.getName(), "1-" + arena.playersInArena.size(), "2-"
                        + arena.maxPlayers);
            }
        } else {
            if (message) {
                MessageHelper.sendMessage(player, MessageKey.ERROR_LEAVE_NOT_IN_ARENA);
            }
            return;
        }

        SignsHandler.updateSigns();
    }

    public static void seekersWin(Arena arena) {
        String cause = "[BlockHunt] Seekers have won " + arena.arenaName;

        List<Player> winners = new ArrayList<>();
        List<Player> losers = new ArrayList<>();

        for (Player player : arena.playersInArena) {

            if (arena.seekers.contains(player)) {

                winners.add(player);

                if (arena.seekersWinCommands != null) {
                    for (String command : arena.seekersWinCommands) {
                        Bukkit.dispatchCommand(Bukkit.getConsoleSender(), command.replaceAll("%player%", player.getDisplayName()));
                    }

                    if (MemoryStorage.shop.getFileConfiguration().get(player.getName() + ".tokens") == null) {
                        MemoryStorage.shop.getFileConfiguration().set(player.getName() + ".tokens", 0);
                        MemoryStorage.shop.save();
                    }
                    int playerTokens = MemoryStorage.shop.getFileConfiguration().getInt(player.getName() + ".tokens");
                    MemoryStorage.shop.getFileConfiguration().set(player.getName() + ".tokens", playerTokens + arena.seekersTokenWin);
                    MemoryStorage.shop.save();

                    MessageHelper.sendMessage(player, MessageKey.NORMAL_ADDED_TOKEN, "amount-" + arena.seekersTokenWin);

                }

            } else {
                losers.add(player);
            }
        }

        EndArenaEvent event = new EndArenaEvent(winners, losers, arena);
        Bukkit.getServer().getPluginManager().callEvent(event);
        stopArena(arena, cause, MessageKey.normal_winSeekers);
    }

    public static void hidersWin(Arena arena) {
        String cause = "[BlockHunt] Hiders have won " + arena.arenaName;

        List<Player> winners = new ArrayList<>();
        List<Player> losers = new ArrayList<>();

        for (Player player : arena.playersInArena) {

            if (arena.seekers.contains(player)) {
                losers.add(player);
            } else {

                winners.add(player);

                if (arena.hidersWinCommands != null) {
                    for (String command : arena.hidersWinCommands) {
                        Bukkit.dispatchCommand(Bukkit.getConsoleSender(), command.replaceAll("%player%", player.getDisplayName()));
                    }

                    if (MemoryStorage.shop.getFileConfiguration().get(player.getName() + ".tokens") == null) {
                        MemoryStorage.shop.getFileConfiguration().set(player.getName() + ".tokens", 0);
                        MemoryStorage.shop.save();
                    }
                    int playerTokens = MemoryStorage.shop.getFileConfiguration().getInt(player.getName() + ".tokens");
                    MemoryStorage.shop.getFileConfiguration().set(player.getName() + ".tokens", playerTokens + arena.hidersTokenWin);
                    MemoryStorage.shop.save();

                    MessageHelper.sendMessage(player, MessageKey.NORMAL_ADDED_TOKEN, "amount-" + arena.hidersTokenWin);
                }
            }

        }

        EndArenaEvent event = new EndArenaEvent(winners, losers, arena);
        Bukkit.getServer().getPluginManager().callEvent(event);
        stopArena(arena, cause, MessageKey.NORMAL_WIN_HIDERS);
    }

    public static void stopArena(Arena arena, String cause, MessageKey message) {
        BlockHunt.plugin.getLogger().log(Level.INFO, cause);
        ArenaHandler.sendMessage(arena, message);
        arena.seekers.clear();
        arena.hiders.clear();

        for (Player player : arena.playersInArena) {
            playerLeaveArena(player, false, false);
            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1, 1);
        }

        arena.gameState = ArenaState.WAITING;
        arena.timer = 0;
        arena.playersInArena.clear();
    }

    public static boolean noPlayersInArenas() {
        // Check if there are any players in any arena (quick way to early exit for event handlers)
        for (Arena arena : MemoryStorage.arenaMap.values()) {
            if (arena.playersInArena.size() > 0) {
                return false;
            }
        }
        return true;
    }

}
