package net.knarcraft.blockhunt.listener;

import net.knarcraft.blockhunt.InventoryHandler;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.SignsHandler;
import net.knarcraft.blockhunt.SolidBlockHandler;
import net.knarcraft.blockhunt.arena.Arena;
import net.knarcraft.blockhunt.arena.ArenaHandler;
import net.knarcraft.blockhunt.arena.ArenaState;
import net.knarcraft.blockhunt.config.ConfigKey;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.config.Permission;
import net.knarcraft.blockhunt.manager.PermissionsManager;
import net.knarcraft.blockhunt.util.MessageHelper;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.block.sign.Side;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.Map;

public class OnPlayerInteractEvent implements Listener {

    @EventHandler(priority = EventPriority.NORMAL, ignoreCancelled = true)
    public void onPlayerInteractEvent(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        Block block = event.getClickedBlock();
        if (PermissionsManager.hasPermission(player, Permission.CREATE, false)) {
            handleWandClick(event, player, block);
        }

        if (event.getAction() == Action.RIGHT_CLICK_BLOCK && event.getClickedBlock() != null &&
                event.getClickedBlock().getState() instanceof Sign sign &&
                SignsHandler.isSign(event.getClickedBlock().getLocation())) {
            handleSignClick(player, sign);
        }

        if (event.getAction() == Action.RIGHT_CLICK_BLOCK || event.getAction() == Action.LEFT_CLICK_BLOCK) {
            if (event.getClickedBlock().getType() != Material.AIR) {
                if (event.getClickedBlock().getType().equals(Material.ENCHANTING_TABLE) || event.getClickedBlock().getType().equals(Material.CRAFTING_TABLE)
                        || event.getClickedBlock().getType().equals(Material.FURNACE) || event.getClickedBlock().getType().equals(Material.CHEST)
                        || event.getClickedBlock().getType().equals(Material.ANVIL) || event.getClickedBlock().getType().equals(Material.ENDER_CHEST)
                        || event.getClickedBlock().getType().equals(Material.JUKEBOX) || block.getRelative(event.getBlockFace()).getType().equals(Material.FIRE)) {
                    for (Arena arena : MemoryStorage.arenaMap.values()) {
                        if (arena.playersInArena.contains(player)) {
                            event.setCancelled(true);
                        }
                    }
                }
            }
        }

        if (event.getAction() == Action.LEFT_CLICK_BLOCK || event.getAction() == Action.LEFT_CLICK_BLOCK) {
            for (Arena arena : MemoryStorage.arenaMap.values()) {
                if (arena.seekers.contains(player)) {
                    for (Player pl : arena.playersInArena) {
                        if (MemoryStorage.hiddenLocation.get(pl) != null) {
                            Block pLoc = event.getClickedBlock();
                            Block moveLocBlock = MemoryStorage.hiddenLocation.get(pl).getBlock();
                            if (moveLocBlock.getX() == pLoc.getX() && moveLocBlock.getY() == pLoc.getY() && moveLocBlock.getZ() == pLoc.getZ()) {
                                MemoryStorage.moveLocation.put(pl, new Location(pl.getWorld(), 0, 0, 0));
                                pl.getWorld().playSound(player.getLocation(), Sound.ENTITY_PLAYER_HURT, 1, 1);
                                SolidBlockHandler.makePlayerUnsolid(pl);
                            }
                        }
                    }
                }
            }
        }

        for (Arena arena : MemoryStorage.arenaMap.values()) {
            if (arena.playersInArena.contains(player) && (arena.gameState.equals(ArenaState.WAITING) || arena.gameState.equals(ArenaState.STARTING))) {
                event.setCancelled(true);
                ItemStack item = player.getInventory().getItemInMainHand();
                if (item.getType() != Material.AIR) {
                    if (item.getItemMeta().getDisplayName() != null) {
                        if (item.getItemMeta().getDisplayName().equals(MessageHelper.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME)))) {
                            Inventory blockChooser = Bukkit.createInventory(null, 36, MessageHelper.replaceAll("\u00A7r" + MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_CHOOSER_V_1_NAME)));
                            if (arena.disguiseBlocks != null) {
                                for (int i = arena.disguiseBlocks.size(); i > 0; i = i - 1) {
                                    blockChooser.setItem(i - 1, arena.disguiseBlocks.get(i - 1));
                                }
                            }

                            player.openInventory(blockChooser);
                        }

                        if (item.getItemMeta().getDisplayName().equals(MessageHelper.replaceAll((String) MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME)))) {
                            Inventory BlockHuntPass = Bukkit.createInventory(null, 9, MessageHelper.replaceAll("\u00A7r" + MemoryStorage.config.get(ConfigKey.SHOP_BLOCK_HUNT_PASS_V_2_NAME)));
                            ItemStack BlockHuntPassSEEKER = new ItemStack(Material.BLUE_WOOL, 1);
                            ItemMeta BlockHuntPassIM = BlockHuntPassSEEKER.getItemMeta();
                            BlockHuntPassIM.setDisplayName(MessageHelper.replaceAll("&eSEEKER"));
                            BlockHuntPassSEEKER.setItemMeta(BlockHuntPassIM);
                            BlockHuntPass.setItem(1, BlockHuntPassSEEKER);

                            ItemStack BlockHuntPassHIDER = new ItemStack(Material.RED_WOOL, 1);
                            BlockHuntPassIM.setDisplayName(MessageHelper.replaceAll("&eHIDER"));
                            BlockHuntPassHIDER.setItemMeta(BlockHuntPassIM);
                            BlockHuntPass.setItem(7, BlockHuntPassHIDER);

                            player.openInventory(BlockHuntPass);
                        }
                    }
                }
            }
        }

    }

    private void handleSignClick(Player player, Sign sign) {
        FileConfiguration fileConfiguration = MemoryStorage.config.getFileConfiguration();
        String signIdentifier = sign.getSide(Side.FRONT).getLine(1);
        if (signIdentifier.equals(MessageHelper.replaceAll(
                fileConfiguration.getStringList(ConfigKey.SIGN_LEAVE.getPath()).get(1)))) {
            if (PermissionsManager.hasPermission(player, Permission.JOIN_SIGN, true)) {
                ArenaHandler.playerLeaveArena(player, true, true);
            }
        } else if (signIdentifier.equals(MessageHelper.replaceAll(
                fileConfiguration.getStringList(ConfigKey.SIGN_SHOP.getPath()).get(1)))) {
            if (PermissionsManager.hasPermission(player, Permission.SHOP, true)) {
                InventoryHandler.openShop(player);
            }
        } else {
            String cleanName = ChatColor.stripColor(ChatColor.translateAlternateColorCodes('&',
                    signIdentifier.toLowerCase()));
            Arena arena = MemoryStorage.arenaMap.get(cleanName);
            if (arena != null && PermissionsManager.hasPermission(player, Permission.JOIN_SIGN, true)) {
                ArenaHandler.playerJoinArena(player, arena.arenaName);
            }
        }
    }

    /**
     * Handles a player clicking a block with the selection wand
     *
     * @param event  <p>The triggered interaction event</p>
     * @param player <p>The player that interacted with a block</p>
     * @param block  <p>The block the player clicked</p>
     */
    private void handleWandClick(PlayerInteractEvent event, Player player, Block block) {
        ItemStack item = player.getInventory().getItemInMainHand();
        if (item.getType() == Material.AIR || item.getItemMeta() == null || !item.getItemMeta().hasDisplayName()) {
            return;
        }

        ItemMeta itemMeta = item.getItemMeta();
        if (!itemMeta.getDisplayName().equals(MessageHelper.replaceAll((String) MemoryStorage.config.get(ConfigKey.WAND_NAME)))) {
            return;
        }

        Action action = event.getAction();
        if (!event.hasBlock() || block == null) {
            return;
        }

        Location location = block.getLocation();
        if (!action.equals(Action.LEFT_CLICK_BLOCK) && !action.equals(Action.RIGHT_CLICK_BLOCK)) {
            return;
        }

        event.setCancelled(true);

        int number;
        Map<Player, Location> locationMap;
        if (action.equals(Action.LEFT_CLICK_BLOCK)) {
            locationMap = MemoryStorage.pos1;
            number = 1;
        } else {
            locationMap = MemoryStorage.pos2;
            number = 2;
        }

        if (locationMap.get(player) == null || !locationMap.get(player).equals(location)) {
            MessageHelper.sendMessage(player, MessageKey.NORMAL_WAND_SET_POSITION, "number-" + number,
                    "x-" + block.getX(), "y-" + block.getY(), "z-" + block.getZ());
            locationMap.put(player, location);
        }
    }

}
