package net.knarcraft.blockhunt.manager;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.config.Permission;
import net.knarcraft.blockhunt.config.PermissionDefault;
import net.knarcraft.blockhunt.util.MessageHelper;
import org.bukkit.entity.Player;

public class PermissionsManager {

    /**
     * Check if a player has the permission. Also checks * related permissions.
     * This permission DOES HAVE TO be registered.
     *
     * @param player     The player who needs the permission.
     * @param permission The permission.
     * @param message    Send a message to the player saying you don't have the
     *                   permission.
     * @return True/False when either the player has the permission.
     */
    public static boolean hasPermission(Player player, Permission permission, Boolean message) {
        PermissionDefault type = permission.type;
        if (player == null) {
            return true;
        }

        if (type == PermissionDefault.ALL) {
            return true;
        } else if (type == PermissionDefault.OP) {
            if (player.isOp()) {
                return true;
            }
        } else if (type == PermissionDefault.ADMIN) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "admin")) {
                return true;
            }
        } else if (type == PermissionDefault.MODERATOR) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "moderator")) {
                return true;
            }
        } else if (type == PermissionDefault.PLAYER) {
            if (player.hasPermission(BlockHunt.plugin.rootPermission + "player")) {
                return true;
            }
        }

        if (player.hasPermission("*")) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + "*")) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + permission.perm)) {
            return true;
        } else if (player.hasPermission(BlockHunt.plugin.rootPermission + permission.perm + ".*")) {
            return true;
        } else {
            if (message) {
                MessageHelper.sendMessage(player, MessageKey.ERROR_NO_PERMISSION);
            }
            return false;
        }
    }

}
